@@ -1,6 +1,16 @@
-# Copyright 2022 Joe Block <jpb@unixorn.net>
+#    Copyright 2022-2023 Joe Block <jpb@unixorn.net>
 #
-# License: Apache 2.0 (see root of the repo)
+#    Licensed under the Apache License, Version 2.0 (the "License");
+#    you may not use this file except in compliance with the License.
+#    You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS,
+#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#    See the License for the specific language governing permissions and
+#    limitations under the License.
 
 import json
 import logging
@@ -538,7 +548,7 @@ class Settings(GenericModel, Generic[EntityType]):
         discovery_prefix: str = "homeassistant"
         """The root of the topic tree where HA is listening for messages"""
         state_prefix: str = "hmd"
-        """The root of the topic tree ha-mqtt-discovery published its state messages"""
+        """The root of the topic tree ha-mqtt-discovery publishes its state messages"""
 
     mqtt: MQTT
     """Connection to MQTT broker"""
@@ -565,6 +575,7 @@ class Discoverable(Generic[EntityType]):
     config_topic: str
     state_topic: str
     availability_topic: str
+    attributes_topic: str
 
     def __init__(
         self, settings: Settings[EntityType], on_connect: Optional[Callable] = None
@@ -607,8 +618,16 @@ class Discoverable(Generic[EntityType]):
         self.state_topic = (
             f"{self._settings.mqtt.state_prefix}/{self._entity_topic}/state"
         )
-        logging.info(f"config_topic: {self.config_topic}")
-        logging.info(f"state_topic: {self.state_topic}")
+
+        # Full topic where we publish our own attributes as JSON messages
+        # Prepend the `state_prefix`, default: `hmd`
+        # e.g. hmd/binary_sensor/mydevice/mysensor
+        self.attributes_topic = (
+            f"{self._settings.mqtt.state_prefix}/{self._entity_topic}/attributes"
+        )
+
+        logger.info(f"config_topic: {self.config_topic}")
+        logger.info(f"state_topic: {self.state_topic}")
         if self._settings.manual_availability:
             # Define the availability topic, using `hmd` topic prefix
             self.availability_topic = (
@@ -638,15 +657,15 @@ wrote_configuration: {self.wrote_configuration}
     def _setup_client(self, on_connect: Optional[Callable] = None) -> None:
         """Create an MQTT client and setup some basic properties on it"""
         mqtt_settings = self._settings.mqtt
-        logging.debug(
+        logger.debug(
             f"Creating mqtt client({mqtt_settings.client_name}) for {mqtt_settings.host}"
         )
         self.mqtt_client = mqtt.Client(mqtt_settings.client_name)
         if mqtt_settings.tls_key:
-            logging.info(f"Connecting to {mqtt_settings.host} with SSL")
-            logging.debug(f"ca_certs={mqtt_settings.tls_ca_cert}")
-            logging.debug(f"certfile={mqtt_settings.tls_certfile}")
-            logging.debug(f"keyfile={mqtt_settings.tls_key}")
+            logger.info(f"Connecting to {mqtt_settings.host} with SSL")
+            logger.debug(f"ca_certs={mqtt_settings.tls_ca_cert}")
+            logger.debug(f"certfile={mqtt_settings.tls_certfile}")
+            logger.debug(f"keyfile={mqtt_settings.tls_key}")
             self.mqtt_client.tls_set(
                 ca_certs=mqtt_settings.tls_ca_cert,
                 certfile=mqtt_settings.tls_certfile,
@@ -655,7 +674,7 @@ wrote_configuration: {self.wrote_configuration}
                 tls_version=ssl.PROTOCOL_TLS,
             )
         else:
-            logging.warning(f"Connecting to {mqtt_settings.host} without SSL")
+            logger.debug(f"Connecting to {mqtt_settings.host} without SSL")
             if mqtt_settings.username:
                 self.mqtt_client.username_pw_set(
                     mqtt_settings.username, password=mqtt_settings.password
@@ -678,10 +697,10 @@ wrote_configuration: {self.wrote_configuration}
         self.mqtt_client.loop_start()
 
     def _state_helper(
-        self, state: Optional[str], topic: Optional[str] = None
+        self, state: Optional[str], topic: Optional[str] = None, retain=True
     ) -> MQTTMessageInfo:
         """
-        Write a state to the given MQTT topic, return the result of client.publish()
+        Write a state to the given MQTT topic, returning the result of client.publish()
         """
         if not self.wrote_configuration:
             logger.debug("Writing sensor configuration")
@@ -692,16 +711,16 @@ wrote_configuration: {self.wrote_configuration}
         logger.debug(f"Writing '{state}' to {topic}")
 
         if self._settings.debug:
-            logging.warning(f"Debug is {self.debug}, skipping state write")
+            logger.debug(f"Debug is {self.debug}, skipping state write")
             return
 
-        message_info = self.mqtt_client.publish(topic, state, retain=True)
+        message_info = self.mqtt_client.publish(topic, state, retain=retain)
         logger.debug(f"Publish result: {message_info}")
         return message_info
 
     def debug_mode(self, mode: bool):
         self.debug = mode
-        logging.warning(f"Set debug mode to {self.debug}")
+        logger.debug(f"Set debug mode to {self.debug}")
 
     def delete(self) -> None:
         """
@@ -716,7 +735,7 @@ wrote_configuration: {self.wrote_configuration}
         """
 
         config_message = ""
-        logging.info(
+        logger.info(
             f"Writing '{config_message}' to topic {self.config_topic} on {self._settings.mqtt.host}"
         )
         self.mqtt_client.publish(self.config_topic, config_message, retain=True)
@@ -733,6 +752,7 @@ wrote_configuration: {self.wrote_configuration}
         # Add the MQTT topics to be discovered by HA
         topics = {
             "state_topic": self.state_topic,
+            "json_attributes_topic": self.attributes_topic,
         }
         # Add availability topic if defined
         if hasattr(self, "availability_topic"):
@@ -748,18 +768,29 @@ wrote_configuration: {self.wrote_configuration}
         """
         config_message = json.dumps(self.generate_config())
 
-        logging.debug(
+        logger.debug(
             f"Writing '{config_message}' to topic {self.config_topic} on {self._settings.mqtt.host}"
         )
         self.wrote_configuration = True
         self.config_message = config_message
 
         if self._settings.debug:
-            logging.warning("Debug mode is enabled, skipping config write.")
+            logger.debug("Debug mode is enabled, skipping config write.")
             return None
 
         return self.mqtt_client.publish(self.config_topic, config_message, retain=True)
 
+    def set_attributes(self, attributes: dict[str, Any]):
+        """Update the attributes of the entity
+
+        Args:
+            attributes: dictionary containing all the attributes that will be set for this entity
+        """
+        # HA expects a JSON object in the attribute topic
+        json_attributes = json.dumps(attributes)
+        logger.debug("Updating attributes: %s", json_attributes)
+        self._state_helper(json_attributes, topic=self.attributes_topic)
+
     def set_availability(self, availability: bool):
         if not hasattr(self, "availability_topic"):
             raise RuntimeError("Manual availability is not configured for this entity!")
@@ -776,7 +807,7 @@ wrote_configuration: {self.wrote_configuration}
 
     def __del__(self):
         """Cleanly shutdown the internal MQTT client"""
-        logging.debug("Shutting down MQTT client")
+        logger.debug("Shutting down MQTT client")
         self.mqtt_client.disconnect()
         self.mqtt_client.loop_stop()
 
@@ -803,6 +834,7 @@ class Subscriber(Discoverable[EntityType]):
             command_callback: Callback function invoked when there is a command
             coming from the MQTT command topic
         """
+
         # Callback invoked when the MQTT connection is established
         def on_client_connected(client: mqtt.Client, *args):
             # Publish this button in Home Assistant
