@@ -1,11 +1,31 @@
 #
-# Copyright 2022 Joe Block <jpb@unixorn.net>
-# License: Apache 2.0
+#    Copyright 2022-2023 Joe Block <jpb@unixorn.net>
+#
+#    Licensed under the Apache License, Version 2.0 (the "License");
+#    you may not use this file except in compliance with the License.
+#    You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS,
+#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#    See the License for the specific language governing permissions and
+#    limitations under the License.
+#
 # Required to define a class itself as type https://stackoverflow.com/a/33533514
 from __future__ import annotations
 import logging
-from typing import Optional
-from ha_mqtt_discoverable import Discoverable, EntityInfo, Subscriber
+from typing import Any, Optional
+
+from ha_mqtt_discoverable import (
+    DeviceInfo,
+    Discoverable,
+    EntityInfo,
+    Subscriber,
+)
+
+logger = logging.getLogger(__name__)
 
 
 class BinarySensorInfo(EntityInfo):
@@ -15,9 +35,9 @@ class BinarySensorInfo(EntityInfo):
     off_delay: Optional[int] = None
     """For sensors that only send on state updates (like PIRs),
     this variable sets a delay in seconds after which the sensor’s state will be updated back to off."""
-    payload_off: str = "on"
+    payload_off: str = "off"
     """Payload to send for the ON state"""
-    payload_on: str = "off"
+    payload_on: str = "on"
     """Payload to send for the OFF state"""
 
 
@@ -61,6 +81,23 @@ class ButtonInfo(EntityInfo):
     """If the published message should have the retain flag on or not"""
 
 
+class DeviceTriggerInfo(EntityInfo):
+    """Information about the device trigger"""
+
+    component: str = "device_automation"
+    automation_type: str = "trigger"
+    """The type of automation, must be ‘trigger’."""
+
+    payload: Optional[str] = None
+    """Optional payload to match the payload being sent over the topic."""
+    type: str
+    """The type of the trigger"""
+    subtype: str
+    """The subtype of the trigger"""
+    device: DeviceInfo
+    """Information about the device this sensor belongs to (required)"""
+
+
 class BinarySensor(Discoverable[BinarySensorInfo]):
     def off(self):
         """
@@ -85,7 +122,7 @@ class BinarySensor(Discoverable[BinarySensorInfo]):
             state_message = self._entity.payload_on
         else:
             state_message = self._entity.payload_off
-        logging.info(
+        logger.info(
             f"Setting {self._entity.name} to {state_message} using {self.state_topic}"
         )
         self._state_helper(state=state_message)
@@ -99,7 +136,7 @@ class Sensor(Discoverable[SensorInfo]):
         Args:
             state(str): What state to set the sensor to
         """
-        logging.info(f"Setting {self._entity.name} to {state} using {self.state_topic}")
+        logger.info(f"Setting {self._entity.name} to {state} using {self.state_topic}")
         self._state_helper(str(state))
 
 
@@ -126,3 +163,30 @@ class Button(Subscriber[ButtonInfo]):
     """Implements an MQTT button:
     https://www.home-assistant.io/integrations/button.mqtt
     """
+
+
+class DeviceTrigger(Discoverable[DeviceTriggerInfo]):
+    """Implements an MWTT Device Trigger
+    https://www.home-assistant.io/integrations/device_trigger.mqtt/
+    """
+
+    def generate_config(self) -> dict[str, Any]:
+        """Publish a custom configuration:
+        since this entity does not provide a `state_topic`, HA expects a `topic` key in the config
+        """
+        config = super().generate_config()
+        # Publish our `state_topic` as `topic`
+        topics = {
+            "topic": self.state_topic,
+        }
+        return config | topics
+
+    def trigger(self, payload: Optional[str] = None):
+        """
+        Generate a device trigger event
+
+        Args:
+            payload: custom payload to send in the trigger topic
+
+        """
+        return self._state_helper(payload, self.state_topic, retain=False)
